# Copyright 2014 Budapest University of Technology and Economics (BME IK)
#
# This file is part of CIRCLE Cloud.
#
# CIRCLE is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# CIRCLE is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along
# with CIRCLE.  If not, see <http://www.gnu.org/licenses/>.

import logging
from os import getenv

from django.utils import timezone
from django.utils.translation import ugettext_noop
from django.core.exceptions import ObjectDoesNotExist

from manager.mancelery import celery
from vm.models import (
    Node, Instance, AzureBuiltinPublisher,
    AzureBuiltinOffer, AzureBuiltinSKU, AzureBuiltinVersion,
    AzureVirtualMachineSize,
)
from azure.common.credentials import ServicePrincipalCredentials
from azure.mgmt.compute import ComputeManagementClient

logger = logging.getLogger(__name__)

SUBSCRIPTION_ID = getenv('SUBSCRIPTION_ID')
CLIENT_ID = getenv('CLIENT_ID')
SECRET = getenv('SECRET')
TENANT = getenv('TENANT')
GROUP_NAME = getenv('GROUP_NAME')
REGION = getenv('REGION')

credentials = ServicePrincipalCredentials(
    client_id = CLIENT_ID,
    secret = SECRET,
    tenant = TENANT,
)

compute_client = ComputeManagementClient(
    credentials,
    SUBSCRIPTION_ID
)

@celery.task(ignore_result=True)
def update_domain_states():
    nodes = Node.objects.filter(enabled=True).all()
    for node in nodes:
        node.update_vm_states()


@celery.task(ignore_result=True)
def garbage_collector(timeout=15):
    """Garbage collector for instances.

    Suspends and destroys expired instances.

    :param timeout: Seconds before TimeOut exception
    :type timeout: int
    """
    now = timezone.now()
    for i in Instance.objects.filter(destroyed_at=None).all():
        if i.time_of_delete and now > i.time_of_delete:
            i.destroy.async(system=True)
            logger.info("Expired instance %d destroyed.", i.pk)
            try:
                i.owner.profile.notify(
                    ugettext_noop('%(instance)s destroyed'),
                    ugettext_noop(
                        'Your instance <a href="%(url)s">%(instance)s</a> '
                        'has been destroyed due to expiration.'),
                    instance=i.name, url=i.get_absolute_url())
            except Exception as e:
                logger.debug('Could not notify owner of instance %d .%s',
                             i.pk, unicode(e))
        elif (i.time_of_suspend and now > i.time_of_suspend and
              i.state == 'RUNNING'):
            i.sleep.async(system=True)
            logger.info("Expired instance %d suspended." % i.pk)
            try:
                i.owner.profile.notify(
                    ugettext_noop('%(instance)s suspended'),
                    ugettext_noop(
                        'Your instance <a href="%(url)s">%(instance)s</a> '
                        'has been suspended due to expiration. '
                        'You can resume or destroy it.'),
                    instance=i.name, url=i.get_absolute_url())
            except Exception as e:
                logger.debug('Could not notify owner of instance %d .%s',
                             i.pk, unicode(e))
        elif i.is_expiring():
            logger.debug("Instance %d expires soon." % i.pk)
            i.notify_owners_about_expiration()
        else:
            logger.debug("Instance %d didn't expire." % i.pk)

@celery.task(ignore_result=True)
def update_azure_vm_sizes():
    logger.info("beginning azure vm size imports")

    vm_sizes = compute_client.virtual_machine_sizes.list(
        REGION,
    )

    for vm_size in vm_sizes:
        try:
            old_vm_size = AzureVirtualMachineSize.objects.get(name=vm_size.name)
            old_vm_size.name = vm_size.name
            old_vm_size.number_of_cores = vm_size.number_of_cores
            old_vm_size.os_disk_size_in_mb = vm_size.os_disk_size_in_mb
            old_vm_size.resource_disk_size_in_mb = \
                vm_size.resource_disk_size_in_mb
            old_vm_size.memory_in_mb = vm_size.memory_in_mb
            old_vm_size.max_data_disk_count = vm_size.max_data_disk_count
            old_vm_size.save()

        except ObjectDoesNotExist:
            new_vm_size = AzureVirtualMachineSize(
                name = vm_size.name,
                number_of_cores = vm_size.number_of_cores,
                os_disk_size_in_mb = vm_size.os_disk_size_in_mb,
                resource_disk_size_in_mb = vm_size.resource_disk_size_in_mb,
                memory_in_mb = vm_size.memory_in_mb,
                max_data_disk_count = vm_size.max_data_disk_count,
            )
            new_vm_size.save()

    logger.info("finished importing azure vm sizes")

@celery.task(ignore_result=True)
def update_azure_builtin_templates():
    logger.info("beginning azure built-in template imports")

    current_publisher_ids = []
    current_offer_ids = []
    current_sku_ids = []
    current_version_ids = []

    # get currently available publishers, offers, skus, and versions
    result_list_pub = compute_client.virtual_machine_images.list_publishers(
        REGION,
    )
    for publisher in result_list_pub:
        current_publisher = AzureBuiltinPublisher(
            azure_id=publisher.id, name=publisher.name,
        )
        current_publisher.save()
        current_publisher_ids.append(current_publisher.azure_id)

        result_list_offers = compute_client.virtual_machine_images.list_offers(
            REGION,
            publisher.name,
        )
        for offer in result_list_offers:
            current_offer = AzureBuiltinOffer(
                azure_id=offer.id, name=offer.name, publisher=current_publisher
            )
            current_offer.save()
            current_offer_ids.append(current_offer.azure_id)
        
            result_list_skus = compute_client.virtual_machine_images.list_skus(
                REGION,
                publisher.name,
                offer.name,
            )
            for sku in result_list_skus:
                current_sku = AzureBuiltinSKU(
                    azure_id=sku.id, name=sku.name, offer=current_offer
                )
                current_sku.save()
                current_sku_ids.append(current_sku.azure_id)
        
                result_list = compute_client.virtual_machine_images.list(
                    REGION,
                    publisher.name,
                    offer.name,
                    sku.name,
                )
                for version in result_list:
                    current_version = AzureBuiltinVersion(
                        azure_id=version.id, name=version.name, sku=current_sku
                    )
                    current_version.save()
                    current_version_ids.append(current_version.azure_id)
    
    # delete old publishers, offers, skus, and versions
    AzureBuiltinPublisher.objects.exclude(
        azure_id__in=current_publisher_ids).delete()
    AzureBuiltinOffer.objects.exclude(
        azure_id__in=current_offer_ids).delete()
    AzureBuiltinSKU.objects.exclude(
        azure_id__in=current_sku_ids).delete()
    AzureBuiltinVersion.objects.exclude(
        azure_id__in=current_version_ids).delete()
    
    logger.info("refreshed azure built-in template imports")